@@ -3,7 +3,7 @@
   error.c -
 
   $Author: matz $
-  $Date: 2005/07/20 01:09:53 $
+  $Date: 2006/07/20 07:04:13 $
   created at: Mon Aug  9 16:11:34 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -303,7 +303,7 @@ VALUE rb_eNoMethodError;
 VALUE rb_eSecurityError;
 VALUE rb_eNotImpError;
 VALUE rb_eNoMemError;
-static VALUE rb_cNameErrorMesg;
+VALUE rb_cNameErrorMesg;
 
 VALUE rb_eScriptError;
 VALUE rb_eSyntaxError;
@@ -1083,7 +1083,7 @@ void
 rb_notimplement()
 {
     rb_raise(rb_eNotImpError,
-	     "the %s() function is unimplemented on this machine",
+	     "%s() function is unimplemented on this machine",
 	     rb_id2name(ruby_frame->last_func));
 }
 
