@@ -53,7 +53,7 @@
 #include <string.h>
 #include <time.h>
 #endif
-#if defined(TM_IN_SYS_TIME) || ! defined(GAWK)
+#if defined(TM_IN_SYS_TIME) || !defined(GAWK) && !defined(_WIN32_WCE)
 #include <sys/types.h>
 #include <sys/time.h>
 #endif
@@ -121,9 +121,13 @@ extern int daylight;
 #ifdef SOLARIS
 extern long timezone, altzone;
 #else
+#ifdef __hpux
+extern long timezone;
+#else
 extern int timezone, altzone;
 #endif
 #endif
+#endif
 
 #undef min	/* just in case */
 
@@ -175,7 +179,8 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 	char *start = s;
 	auto char tbuf[100];
 	long off;
-	int i, w, y;
+	int i, w;
+	long y;
 	static short first = 1;
 #ifdef POSIX_SEMANTICS
 	static char *savetz = NULL;
@@ -188,6 +193,11 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 	extern char *timezone();
 	struct timeval tv;
 	struct timezone zone;
+#else
+#ifdef __hpux
+	struct timeval tv;
+	struct timezone zone;
+#endif
 #endif /* HAVE_TZNAME */
 #endif /* HAVE_TM_NAME */
 #endif /* HAVE_TM_ZONE */
@@ -378,7 +388,7 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 			break;
 
 		case 'Y':	/* year with century */
-			sprintf(tbuf, "%d", 1900 + timeptr->tm_year);
+			sprintf(tbuf, "%ld", 1900L + timeptr->tm_year);
 			break;
 
 #ifdef MAILHEADER_EXT
@@ -417,7 +427,12 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 			 * Systems with tzname[] probably have timezone as
 			 * secs west of GMT.  Convert to mins east of GMT.
 			 */
+#ifdef __hpux
+			gettimeofday(&tv, &zone);
+			off = -zone.tz_minuteswest;
+#else
 			off = -(daylight ? timezone : altzone) / 60;
+#endif
 #else /* !HAVE_TZNAME */
 			gettimeofday(&tv, &zone);
 			off = -zone.tz_minuteswest;
@@ -430,7 +445,7 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 			} else {
 				tbuf[0] = '+';
 			}
-			sprintf(tbuf+1, "%02d%02d", off/60, off%60);
+			sprintf(tbuf+1, "%02u%02u", (unsigned)off/60, (unsigned)off%60);
 			break;
 #endif /* MAILHEADER_EXT */
 
@@ -507,10 +522,10 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 
 #ifdef VMS_EXT
 		case 'v':	/* date as dd-bbb-YYYY */
-			sprintf(tbuf, "%2d-%3.3s-%4d",
+			sprintf(tbuf, "%2d-%3.3s-%4ld",
 				range(1, timeptr->tm_mday, 31),
 				months_a[range(0, timeptr->tm_mon, 11)],
-				timeptr->tm_year + 1900);
+				timeptr->tm_year + 1900L);
 			for (i = 3; i < 6; i++)
 				if (islower(tbuf[i]))
 					tbuf[i] = toupper(tbuf[i]);
@@ -520,7 +535,7 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 
 #ifdef POSIX2_DATE
 		case 'C':
-			sprintf(tbuf, "%02d", (timeptr->tm_year + 1900) / 100);
+			sprintf(tbuf, "%02ld", (timeptr->tm_year + 1900L) / 100);
 			break;
 
 
@@ -554,16 +569,16 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 			 */
 			w = iso8601wknum(timeptr);
 			if (timeptr->tm_mon == 11 && w == 1)
-				y = 1900 + timeptr->tm_year + 1;
+				y = 1900L + timeptr->tm_year + 1;
 			else if (timeptr->tm_mon == 0 && w >= 52)
-				y = 1900 + timeptr->tm_year - 1;
+				y = 1900L + timeptr->tm_year - 1;
 			else
-				y = 1900 + timeptr->tm_year;
+				y = 1900L + timeptr->tm_year;
 
 			if (*format == 'G')
-				sprintf(tbuf, "%d", y);
+				sprintf(tbuf, "%ld", y);
 			else
-				sprintf(tbuf, "%02d", y % 100);
+				sprintf(tbuf, "%02ld", y % 100);
 			break;
 #endif /* ISO_DATE_EXT */
 		default:
@@ -594,10 +609,10 @@ out:
 #ifndef __STDC__
 static int
 isleap(year)
-int year;
+long year;
 #else
 static int
-isleap(int year)
+isleap(long year)
 #endif
 {
 	return ((year % 4 == 0 && year % 100 != 0) || year % 400 == 0);
@@ -688,7 +703,7 @@ iso8601wknum(const struct tm *timeptr)
 			dec31ly.tm_mon = 11;
 			dec31ly.tm_mday = 31;
 			dec31ly.tm_wday = (jan1day == 0) ? 6 : jan1day - 1;
-			dec31ly.tm_yday = 364 + isleap(dec31ly.tm_year + 1900);
+			dec31ly.tm_yday = 364 + isleap(dec31ly.tm_year + 1900L);
 			weeknum = iso8601wknum(& dec31ly);
 #endif
 		}
