@@ -2,8 +2,8 @@
 
   sprintf.c -
 
-  $Author: akr $
-  $Date: 2005/12/12 15:09:50 $
+  $Author: matz $
+  $Date: 2006/07/28 16:27:42 $
   created at: Fri Oct 15 10:39:26 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -13,6 +13,7 @@
 **********************************************************************/
 
 #include "ruby.h"
+#include "re.h"
 #include <ctype.h>
 #include <math.h>
 
@@ -81,6 +82,7 @@ sign_bits(base, p)
 #define FSPACE 16
 #define FWIDTH 32
 #define FPREC  64
+#define FPREC0 128
 
 #define CHECK(l) do {\
     while (blen + (l) >= bsiz) {\
@@ -109,9 +111,7 @@ sign_bits(base, p)
 #define GETNTHARG(nth) \
     ((nth >= argc) ? (rb_raise(rb_eArgError, "too few arguments"), 0) : argv[nth])
 
-#define GETASTER(val) do { \
-    t = p++; \
-    n = 0; \
+#define GETNUM(n, val) \
     for (; p < end && ISDIGIT(*p); p++) { \
 	int next_n = 10 * n + (*p - '0'); \
         if (next_n / 10 != n) {\
@@ -121,7 +121,12 @@ sign_bits(base, p)
     } \
     if (p >= end) { \
 	rb_raise(rb_eArgError, "malformed format string - %%*[0-9]"); \
-    } \
+    }
+
+#define GETASTER(val) do { \
+    t = p++; \
+    n = 0; \
+    GETNUM(n, val); \
     if (*p == '$') { \
 	tmp = GETPOSARG(n); \
     } \
@@ -212,7 +217,12 @@ sign_bits(base, p)
  *        s   | Argument is a string to be substituted. If the format
  *            | sequence contains a precision, at most that many characters
  *            | will be copied.
- *        u   | Treat argument as an unsigned decimal number.
+ *        u   | Treat argument as an unsigned decimal number. Negative integers
+ *            | are displayed as a 32 bit two's complement plus one for the
+ *            | underlying architecture; that is, 2 ** 32 + n.  However, since
+ *            | Ruby has no inherent limit on bits used to represent the
+ *            | integer, this value is preceded by two dots (..) in order to
+ *            | indicate a infinite number of leading sign bits.
  *        X   | Convert argument as a hexadecimal number using uppercase
  *            | letters. Negative numbers will be displayed with two
  *            | leading periods (representing an infinite string of
@@ -229,6 +239,7 @@ sign_bits(base, p)
  *     sprintf("%1$*2$s %2$d %1$s", "hello", 8)   #=> "   hello 8 hello"
  *     sprintf("%1$*2$s %2$d", "hello", -8)       #=> "hello    -8"
  *     sprintf("%+g:% g:%-g", 1.23, 1.23, 1.23)   #=> "+1.23: 1.23:1.23"
+ *     sprintf("%u", -123)                        #=> "..4294967173"
  */
 
 VALUE
@@ -250,6 +261,21 @@ rb_f_sprintf(argc, argv)
     VALUE tmp;
     VALUE str;
 
+#define CHECK_FOR_WIDTH(f)				 \
+    if ((f) & FWIDTH) {					 \
+	rb_raise(rb_eArgError, "width given twice");	 \
+    }							 \
+    if ((f) & FPREC0) {					 \
+	rb_raise(rb_eArgError, "width after precision"); \
+    }
+#define CHECK_FOR_FLAGS(f)				 \
+    if ((f) & FWIDTH) {					 \
+	rb_raise(rb_eArgError, "flag after width");	 \
+    }							 \
+    if ((f) & FPREC0) {					 \
+	rb_raise(rb_eArgError, "flag after precision"); \
+    }
+
     fmt = GETNTHARG(0);
     if (OBJ_TAINTED(fmt)) tainted = 1;
     StringValue(fmt);
@@ -285,26 +311,31 @@ rb_f_sprintf(argc, argv)
 	    break;
 
 	  case ' ':
+	    CHECK_FOR_FLAGS(flags);
 	    flags |= FSPACE;
 	    p++;
 	    goto retry;
 
 	  case '#':
+	    CHECK_FOR_FLAGS(flags);
 	    flags |= FSHARP;
 	    p++;
 	    goto retry;
 
 	  case '+':
+	    CHECK_FOR_FLAGS(flags);
 	    flags |= FPLUS;
 	    p++;
 	    goto retry;
 
 	  case '-':
+	    CHECK_FOR_FLAGS(flags);
 	    flags |= FMINUS;
 	    p++;
 	    goto retry;
 
 	  case '0':
+	    CHECK_FOR_FLAGS(flags);
 	    flags |= FZERO;
 	    p++;
 	    goto retry;
@@ -312,16 +343,7 @@ rb_f_sprintf(argc, argv)
 	  case '1': case '2': case '3': case '4':
 	  case '5': case '6': case '7': case '8': case '9':
 	    n = 0;
-	    for (; p < end && ISDIGIT(*p); p++) {
-		int next_n = 10 * n + (*p - '0');
-		if (next_n / 10 != n) {
-		    rb_raise(rb_eArgError, "width too big");
-		}
-		n = 10 * n + (*p - '0');
-	    }
-	    if (p >= end) {
-		rb_raise(rb_eArgError, "malformed format string - %%[0-9]");
-	    }
+	    GETNUM(n, width);
 	    if (*p == '$') {
 		if (nextvalue != Qundef) {
 		    rb_raise(rb_eArgError, "value given twice - %d$", n);
@@ -330,15 +352,13 @@ rb_f_sprintf(argc, argv)
 		p++;
 		goto retry;
 	    }
+	    CHECK_FOR_WIDTH(flags);
 	    width = n;
 	    flags |= FWIDTH;
 	    goto retry;
 
 	  case '*':
-	    if (flags & FWIDTH) {
-		rb_raise(rb_eArgError, "width given twice");
-	    }
-
+	    CHECK_FOR_WIDTH(flags);
 	    flags |= FWIDTH;
 	    GETASTER(width);
 	    if (width < 0) {
@@ -349,10 +369,10 @@ rb_f_sprintf(argc, argv)
 	    goto retry;
 
 	  case '.':
-	    if (flags & FPREC) {
+	    if (flags & FPREC0) {
 		rb_raise(rb_eArgError, "precision given twice");
 	    }
-	    flags |= FPREC;
+	    flags |= FPREC|FPREC0;
 
 	    prec = 0;
 	    p++;
@@ -365,17 +385,12 @@ rb_f_sprintf(argc, argv)
 		goto retry;
 	    }
 
-	    for (; p < end && ISDIGIT(*p); p++) {
-		prec = 10 * prec + (*p - '0');
-	    }
-	    if (p >= end) {
-		rb_raise(rb_eArgError, "malformed format string - %%.[0-9]");
-	    }
+	    GETNUM(prec, precision);
 	    goto retry;
 
 	  case '\n':
-	    p--;
 	  case '\0':
+	    p--;
 	  case '%':
 	    if (flags != FNONE) {
 		rb_raise(rb_eArgError, "illegal format character - %%");
@@ -413,6 +428,23 @@ rb_f_sprintf(argc, argv)
 			len = prec;
 		    }
 		}
+		{
+		    char *s, *send;
+		    long l;
+
+			s = RSTRING(str)->ptr;
+			send = s + RSTRING(str)->len;
+			l = 0;
+			while (s < send) {
+			    long n = mbclen(*s);
+			    if (l + n > len) {
+				len = l;
+				break;
+			    }
+			    l += n;
+			    s += n;
+			}
+		}
 		if (flags&FWIDTH) {
 		    if (width > len) {
 			CHECK(width);
@@ -447,7 +479,7 @@ rb_f_sprintf(argc, argv)
 	    {
 		volatile VALUE val = GETARG();
 		char fbuf[32], nbuf[64], *s, *t;
-		char *prefix = 0;
+		const char *prefix = 0;
 		int sign = 0;
 		char sc = 0;
 		long v = 0;
@@ -524,6 +556,7 @@ rb_f_sprintf(argc, argv)
 		  default:
 		    base = 10; break;
 		}
+
 		if (!bignum) {
 		    if (base == 2) {
 			val = rb_int2big(v);
@@ -608,27 +641,24 @@ rb_f_sprintf(argc, argv)
 		if (*s == '-') {
 		    if (base == 10) {
 			rb_warning("negative number for %%u specifier");
-			s++;
 		    }
-		    else {
-			remove_sign_bits(++s, base);
-			tmp = rb_str_new(0, 3+strlen(s));
-			t = RSTRING(tmp)->ptr;
-			if (!(flags&(FPREC|FZERO))) {
-			    strcpy(t, "..");
-			    t += 2;
-			}
-			switch (base) {
-			  case 16:
-			    if (s[0] != 'f') strcpy(t++, "f"); break;
-			  case 8:
-			    if (s[0] != '7') strcpy(t++, "7"); break;
-			  case 2:
-			    if (s[0] != '1') strcpy(t++, "1"); break;
-			}
-			strcpy(t, s);
-			bignum = 2;
+		    remove_sign_bits(++s, base);
+		    tmp = rb_str_new(0, 3+strlen(s));
+		    t = RSTRING(tmp)->ptr;
+		    if (!(flags&(FPREC|FZERO))) {
+			strcpy(t, "..");
+			t += 2;
 		    }
+		    switch (base) {
+		      case 16:
+			if (s[0] != 'f') strcpy(t++, "f"); break;
+		      case 8:
+			if (s[0] != '7') strcpy(t++, "7"); break;
+		      case 2:
+			if (s[0] != '1') strcpy(t++, "1"); break;
+		    }
+		    strcpy(t, s);
+		    bignum = 2;
 		}
 		s = RSTRING(tmp)->ptr;
 
@@ -672,7 +702,7 @@ rb_f_sprintf(argc, argv)
 		else {
 		    char c;
 
-		    if (bignum && !RBIGNUM(val)->sign)
+		    if (!sign && bignum && !RBIGNUM(val)->sign)
 			c = sign_bits(base, p);
 		    else
 			c = '0';
