@@ -3,7 +3,7 @@
   struct.c -
 
   $Author: matz $
-  $Date: 2005/07/19 14:58:59 $
+  $Date: 2006/02/02 07:18:10 $
   created at: Tue Mar 22 18:44:30 JST 1995
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -252,7 +252,7 @@ rb_struct_define(name, va_alist)
     ary = rb_ary_new();
 
     va_init_list(ar, name);
-    while (mem = va_arg(ar, char*)) {
+    while ((mem = va_arg(ar, char*)) != 0) {
 	ID slot = rb_intern(mem);
 	rb_ary_push(ary, ID2SYM(slot));
     }
@@ -557,8 +557,9 @@ rb_struct_init_copy(copy, s)
     if (!rb_obj_is_instance_of(s, rb_obj_class(copy))) {
 	rb_raise(rb_eTypeError, "wrong argument class");
     }
-    RSTRUCT(copy)->ptr = ALLOC_N(VALUE, RSTRUCT(s)->len);
-    RSTRUCT(copy)->len = RSTRUCT(s)->len;
+    if (RSTRUCT(copy)->len != RSTRUCT(s)->len) {
+	rb_raise(rb_eTypeError, "struct size mismatch");
+    }
     MEMCPY(RSTRUCT(copy)->ptr, RSTRUCT(s)->ptr, VALUE, RSTRUCT(copy)->len);
 
     return copy;
@@ -728,21 +729,15 @@ rb_struct_values_at(argc, argv, s)
 
 /*
  *  call-seq:
- *     struct.select(fixnum, ... )   => array
  *     struct.select {|i| block }    => array
  *  
- *  The first form returns an array containing the elements in
- *  <i>struct</i> corresponding to the given indices. The second
- *  form invokes the block passing in successive elements from
+ *  Invokes the block passing in successive elements from
  *  <i>struct</i>, returning an array containing those elements
  *  for which the block returns a true value (equivalent to
  *  <code>Enumerable#select</code>).
  *     
  *     Lots = Struct.new(:a, :b, :c, :d, :e, :f)
  *     l = Lots.new(11, 22, 33, 44, 55, 66)
- *     l.select(1, 3, 5)               #=> [22, 44, 66]
- *     l.select(0, 2, 4)               #=> [11, 33, 55]
- *     l.select(-1, -3, -5)            #=> [66, 44, 22]
  *     l.select {|v| (v % 2).zero? }   #=> [22, 44, 66]
  */
 
